#! /usr/bin/python

"""Commandline interface for the Ishi to SGF converter.

This module offers the commandline userinterface for the ishi2sgf converter.

Copyright (c) 2003-2005 PANDANET Inc.
"""

__version__ = "$Id: ishi_commandline.py 198 2005-02-05 19:29:32Z peter $"
__author__  =  "Peter Strempel"

import getopt, sys, os, fnmatch, ishi, __init__
from glGo.__init__ import sgftools_version
from string import lower

def convert_ext(filename):
    """Replace .go suffix with .sgf."""
    return os.path.splitext(filename)[0] + ".sgf"

def convert(infile, outfile=None):
    """Convert a file or directory from Ishi to SGF.

    If infile is a directory, all .go files within the directory are converted.
    Subdirectories are recursively converted as well..

    Keywork arguments:
    infile  -- Input filename or directory
    outfile -- Output filename or directory (default None)

    Returns:
    Number of converted files

    """
    global game

    # Single file
    if not os.path.isdir(infile):
        if not fnmatch.fnmatch(lower(infile), '*.go'):
            return 0
        if not outfile:
            outfile = convert_ext(infile)
        res = ishi.convertIshi(infile, outfile)
        if res == 0:
            return 1
        else:
            return 0

    # Directory
    else:
        try:
            files = os.listdir(infile)
        except OSError:
            if __init__.info_level >= 1:
                print 'Failed to read from ' + infile
            return 0
        counter = 0
        for f in files:
            fname = os.path.join(infile, f)
            # Recursive directory
            if os.path.isdir(fname):
                counter += convert(fname, outfile)
                continue
            if not fnmatch.fnmatch(lower(f), '*.go'):
                continue
            if outfile:
                of = os.path.abspath(os.path.join(outfile, convert_ext(fname)))
                if not os.path.exists(os.path.dirname(of)):
                    os.makedirs(os.path.dirname(of))
                    if __init__.debug_level >= 1:
                        print "Created dir " + os.path.dirname(of)
            else:
                of = convert_ext(fname)
            game = []
            res = ishi.convertIshi(fname, of)
            if res == 0:
                counter += 1
        return counter

def usage():
    """Show commandline usage."""
    print "Usage is: ishi2sgf [-o <outfile>] <infile>"

def version():
    """Print application name and version."""
    print "%s %s" % (__init__.packagename, sgftools_version)
    print """
Written by Peter Strempel
Copyright (c) 2003-2005 PANDANET Inc."""

def main():
    """Main entry method for the ishi2sgf.py script."""
    try:
        opts, args = getopt.getopt(sys.argv[1:],
                                   "hvo:",
                                   ["help", "version", "outfile="])
    except getopt.GetoptError:
        usage()
        sys.exit(2)

    outfile = None

    for o, a in opts:
        if o in ("-h", "--help"):
            usage()
            sys.exit(0)
        if o in ("-v", "--version"):
            version()
            sys.exit(0)
        if o in ("-o", "--outfile"):
            outfile = a

    if not args:
        usage()
        sys.exit(2)

    c = 0
    for infile in args:
        if not outfile and not os.path.isdir(infile):
            of = convert_ext(infile)
        else:
            of = outfile
        res = convert(infile, of)
        if type(res) == int:
            c += res
    print "Converted %d files." % c
    sys.exit(0)


#
# Debug stuff
#

## if __name__ == '__main__':
##     main()
