#! /usr/bin/python

#
# $Id: ugf2sgf_converter.py 198 2005-02-05 19:29:32Z peter $
#
# Copyright (c) 2003-2005 PANDANET Inc.
#

import sys, os, fnmatch, getopt, __init__
from glGo.__init__ import sgftools_version
import ugf_parser
from ugf_parser import STONE_UNDEFINED, STONE_WHITE, STONE_BLACK, convertUGF
from string import replace, rstrip

class Move:
    def __init__(self, x, y, col):
        self.x = x
        self.y = y
        self.color = col
        self.comment = None
        self.marks = []

    def MoveToSGF(self, is_root=False):
        if not is_root:
            str = ";"
        else:
            str = ""  # No ";" for root node

        if self.color == STONE_BLACK:
            c = "B"
        else:
            c = "W"

        if self.x > 0 and self.y > 0:
            # Write something like 'B[aa]'
            str += "%s[%s%s]" % (c, chr(96 + self.x), chr(96 + self.y))
        elif (self.x == -1 and self.y == -1) or (self.x == 20 and self.y == 20):
            # Pass: 'B[tt]'
            str += "%s[tt]", c

        # Save marks, if we have any
        if self.marks:
            sLB = "LB"
            for m in self.marks:
                sLB += "[%s%s:%s]" % (chr(96 + m[0][0]), chr(96 + m[0][1]), m[1])
            str += sLB

        # Add comment, if we have one
        if self.comment:
            # Work on copy of original comment
            s = self.comment

            # Replace "\" with "\\"
            s = replace(s, "\\", "\\\\")

            # Replace "]" with "\]"
            s = replace(s, "]", "\\]")

            # Replace ";" with "\;"
            s = replace(s, ";", "\\;")

            # Remove trailing "\n" and whitespaces if we have any
            s = rstrip(s)

            str += "C[%s]" % s

        # print "SAVE MOVE: " + str
        return str

game = []

def do_move(move):
    game.append(Move(move[0][0], move[0][1], move[1]))

def do_comment(comment):
    if game:
        game[len(game)-1].comment = comment

def do_mark(pos, text):
    if game:
        game[len(game)-1].marks.append((pos, text))

def saveSGF(filename):
    if not filename:
        print "No filename given."
        return -1
    if not game:
        print "Error saving SGF: No moves!"
        return -1

    # Create output file
    try:
        file = open(filename, "w")
    except IOError, (errno, strerr):
        print "Error writing SGF to file '%s': %s:" % (filename, strerr)
        return -1

    # Write game header to root node
    writeGameHeader(file)

    # Write moves.
    # Unlike the C++ SGF writer, we don't need to traverse as we don't have a tree
    is_root = True
    for m in game:
        file.write(m.MoveToSGF(is_root) + "\n")
        if is_root:
            is_root = False
    file.write(")\n")

    file.close()
    return 0

def writeGameHeader(file):
    data = ugf_parser.headers

    s = ""

    # We play Go, we use FF 4
    s = "(;GM[1]FF[4]"

    # Charset TODO
    # s += "CA[" + data.charset + "]" +

    # Application name : Version
    s += "AP[%s:%s]" % (__init__.packagename , sgftools_version)

    # We show vars of current node
    s += "ST[1]\n"

    # Game Name
    if data.has_key("Title"):
        s += "GN[" + data["Title"] + "]"

    # Board size
    if data.has_key("Size"):
        sz = int(data["Size"])
    else:
        sz = 19
    s += "\nSZ[%d]" % sz

    # Handicap
    if data.has_key("Hdcp"):
        h = int(data["Hdcp"])
        if h >= 2:
            s += "HA[%d]" % h

    # Komi
    if data.has_key("Komi"):
        k = data["Komi"]
    else:
        k = "0.0"
    s += "KM[%s]\n" % k

    # White name
    if data.has_key("PlayerW"):
        s += "PW[%s]" % data["PlayerW"]

    # White rank
    if data.has_key("WhiteRank"):
        s += "WR[%s]" % data["WhiteRank"]

    # Black name
    if data.has_key("PlayerB"):
        s += "PB[%s]" % data["PlayerB"]

    # Black rank
    if data.has_key("BlackRank"):
        s += "BR[%s]" % data["BlackRank"]

    # Result
    if data.has_key("Winner"):
        s += "RE[%s]" % data["Winner"]

    # Time
    # NOT IMPLEMENTED

    # Date
    if data.has_key("Date"):
        s += "DT[%s]" % data["Date"]

    # Place
    if data.has_key("Place"):
        s += "PC[%s]" % data["Place"]

    # Copyright
    if data.has_key("Copyright"):
        s += "CP[%s]" % data["Copyright"]

    s += "\n"
    file.write(s)

def convert_ext(filename):
    return os.path.splitext(filename)[0] + ".sgf"

def convert(infile, outfile=None):
    global game

    # Single file
    if not os.path.isdir(infile):
        if not fnmatch.fnmatch(infile, '*.ug[fi]'):
            return None
        if not outfile:
            outfile = convert_ext(infile)
        game = []
        return convertUGF(infile, outfile)

    # Directory
    else:
        try:
            files = os.listdir(infile)
        except OSError:
            if __init__.info_level >= 1:
                print 'Failed to read from ' + infile
            return None
        counter = 0
        for f in files:
            fname = os.path.join(infile, f)
            # Recursive directory
            if os.path.isdir(fname):
                counter += convert(fname, outfile)
                continue
            if not fnmatch.fnmatch(f, '*.ug[fi]'):
                continue
            if outfile:
                of = os.path.abspath(os.path.join(outfile, convert_ext(fname)))
                if not os.path.exists(os.path.dirname(of)):
                    os.makedirs(os.path.dirname(of))
                    if __init__.debug_level >= 1:
                        print "Created dir " + os.path.dirname(of)
            else:
                of = convert_ext(fname)
            game = []
            counter += convertUGF(fname, of)
        return counter

def usage():
    print "Usage is: ugf2sgf [-o <outfile>] <infile>"

def version():
    print "%s %s" % (__init__.packagename, sgftools_version)
    print """
Written by Peter Strempel
Copyright (c) 2003-2005 PANDANET Inc."""

def main():
    try:
        opts, args = getopt.getopt(sys.argv[1:],
                                   "hvo:",
                                   ["help", "version", "outfile="])
    except getopt.GetoptError:
        usage()
        sys.exit(2)

    outfile = None

    for o, a in opts:
        if o in ("-h", "--help"):
            usage()
            sys.exit(0)
        if o in ("-v", "--version"):
            version()
            sys.exit(0)
        if o in ("-o", "--outfile"):
            outfile = a

    if not args:
        usage()
        sys.exit(2)

    c = 0
    for infile in args:
        if not outfile and not os.path.isdir(infile):
            of = convert_ext(infile)
        else:
            of = outfile
        res = convert(infile, of)
        if type(res) == int:
            c += res
    print "Converted %d files." % c
    sys.exit(0)


#
# Debug stuff
#

## if __name__ == '__main__':
##     main()
