#! /usr/bin/python

"""Utilities for the glGo python package.

The functions in this module are used by several modules within the glGo python package.

Copyright (c) 2003, 2004 Peter Strempel
"""

__version__ = "$Revision: 1.4 $"
__author__  =  "Peter Strempel"

import __init__, os
from string import join, replace, strip

def loadFile(fileName):
    """Load a file.

    Keyword arguments:
    fileName -- File to load

    Returns:
    string containing the file content or None if an error occured

    """
    try:
        file = open(fileName, 'r')
        list = []
        if os.name != 'java':
            fun = file.xreadlines
        else:
            # Jython has no xreadlines
            fun = file.readlines
        for line in fun():
            list.append(line)
        file.close()
        return join(list, '')
    except IOError, (errno, strerr):
        if __init__.info_level >= 1:
            print 'IOError: Failed to read from file ' + fileName
            print strerr
        return None
    except:
        if __init__.info_level >= 1:
            print 'Failed to read from file ' + fileName
        return None

def escapeString(s):
    """Escape some characters for SGF writing.

    Returns:
    Escaped string

    """
    # Replace "\" with "\\"
    s = replace(s, "\\", "\\\\")

    # Replace "]" with "\]"
    s = replace(s, "]", "\\]")

    # SGF spec: "only if used in compose data type)"
    # Replace ":" with "\:"
    # s = replace(s, ":", "\\:");

    # Remove trailing "\n" and whitespaces if we have any
    return s.strip()

_is_darwin = None

def is_darwin():
    """Check if running on Mac OS X."""

    # Cache and reuse the value.
    global _is_darwin
    if _is_darwin:
        return _is_darwin

    if os.name != 'posix':
        _is_darwin = False
        return _is_darwin

    try:
        import MacOS
        if MacOS.runtimemodel == "macho":
            _is_darwin = True   # OS x
        else:
            _is_darwin = False  # Classic
    except ImportError:
        _is_darwin = False     # Not Mac

    return _is_darwin
